/*
* Copyright (c) 2007, [AEGL, UNL] Paladin Vent
* All rights reserved.
*
* Additional authors:
*    [CE] Mirida (Multithreading)
*    [CORE., AUS C] Verite Rendition (Various, General Upkeep: Aug 2007 to Present)
*    [MOP] Calistra "Draekas" Darkwater (Multiple-Instance Name placement)
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the United Legion nor the
*       names of its contributors may be used to endorse or promote products
*       derived from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY [AEGL, UNL] Paladin Vent ``AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
* DISCLAIMED. IN NO EVENT SHALL [AEGL, UNL] Paladin Vent BE LIABLE FOR ANY
* DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
* (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

import java.awt.image.BufferedImage;
import java.awt.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.geom.Rectangle2D;
//import java.text.DateFormat;
import java.text.SimpleDateFormat;
//import java.util.Date;
//import java.text.ParseException;
import java.util.*;
import java.sql.*;
//import java.io.*;

public class Map extends Frame implements Runnable {
    private final int quarternizer = 1; //Unknown, but it's 1 so who cares
	private final int sampleRate = 8; //Sample rate for text placement algorithm, samples every sampleRate pixels
    public final int sWidth=900*2, sHeight=1024*2, sX=180, sY=0, sThreads = 2; //Map width, height, unknown, unknown, number of render threads(see instructions)
    BufferedImage img=new BufferedImage(sWidth, sHeight, BufferedImage.TYPE_INT_ARGB);
    Graphics2D g=img.createGraphics();
    private final double scale=4.8445284569785E17/((sHeight-20)/2.0); //Scaling factor
    private Hashtable<Integer, Alliance> alliances=new Hashtable<Integer, Alliance>(); //Hashtable of all of the alliances
    private Vector<Alliance> alliancesV=new Vector<Alliance>(); //Vector of all of the on-map alliances for when we print names
    private Hashtable<Integer, SolarSystem> systems=new Hashtable<Integer, SolarSystem>(); //Hashtable of the solar systems
    private Vector<SolarSystem> systemsSov=new Vector<SolarSystem>(); //Vector of all the systems with sov claims
    private Hashtable<SolarSystem, Vector> jumpsTable = new Hashtable<SolarSystem, Vector>(); //Hashtable of all the stargates for distributing inflence
    private Vector<Jump> jumpsV=new Vector<Jump>(); //Vector of jumps
    private Vector<Color> colorTable=new Vector<Color>(); //Colortable
    private Connection db;
    private final String soveregntyDate;
    private rowState rowS=new rowState();
    private BlockingIntStack intStack = new BlockingIntStack(sThreads);
	private Alliance[][] sovMap = new Alliance[sWidth/sampleRate][sHeight/sampleRate]; //Matrix for text name placement
	private Vector titleList = new Vector();
	private long time;

    public static void main(String[]args){
        //Initialize calendar to retrieve date
    	
    	Calendar cal = Calendar.getInstance();
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        String dateString = format.format(cal.getTime());
        //Initialize SQL connection
        Connection db;
        try{
            DriverManager.registerDriver(new com.mysql.jdbc.Driver());
            //MySQL connection string, you'll want to set a password
            db=DriverManager.getConnection("jdbc:mysql://localhost/eve", "root", "root");
            
            new Map(db, dateString);
        }catch(Exception ex){
            ex.printStackTrace();
        }
    }

    public Map(Connection db, String date){
        soveregntyDate=date;
        this.db = db;
        time = System.currentTimeMillis();
        this.addWindowListener(new WindowAdapter(){
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
        try{
            //load alliances
            Statement st=db.createStatement();
            ResultSet rs=st.executeQuery("SELECT id, name, color FROM evealliances ORDER BY onMap DESC, name DESC");
            while(rs.next()){
                Alliance al=new Alliance();
                al.id=rs.getInt(1);
                al.name=rs.getString(2);
                String color=rs.getString(3);
                if(color.length()==6){
                    int r=Integer.parseInt(color.substring(0,2),16);
                    int g=Integer.parseInt(color.substring(2,4),16);
                    int b=Integer.parseInt(color.substring(4,6),16);
                    if(r+g+b>=128){
                        al.setColor(new Color(r,g,b));
                    }
                    //al.color=nextColor();
                }
                alliances.put(Integer.valueOf(al.id), al);
                alliancesV.addElement(al);
            }
            rs.close();
            st.close();

            //load systems
            g.setColor(new Color(0, 0, 0x0));
            g.fillRect(0,0,sWidth,sHeight);
            st=db.createStatement();
            rs=st.executeQuery("SELECT x, z, solarSystemID, allianceID, stantion, constellationID, regionID, sovereigntylevel, constellationSov FROM mapsolarsystems");
            while(rs.next()){
                SolarSystem ss=new SolarSystem();
                ss.x=(int)(rs.getDouble(1)/scale)+sWidth/2+sX;
                ss.y=(int)(rs.getDouble(2)/scale)+sHeight/2+sY;
                ss.id=rs.getInt(3);
                ss.constelationID=rs.getInt(6);
                ss.RegionID=rs.getInt(7);
                ss.sovLevel=rs.getInt(8);
                ss.conSov=rs.getInt(9);
                ss.sovereignty=(Alliance)alliances.get(Integer.valueOf(rs.getInt(4)));
                ss.stantion=rs.getInt(5)==1;
                systems.put(Integer.valueOf(ss.id), ss);
                if(ss.sovereignty!=null)systemsSov.addElement(ss);
            }
            rs.close();
            st.close();

            //load jumps
            st=db.createStatement();
            rs=st.executeQuery("SELECT fromSolarSystemID, toSolarSystemID FROM mapsolarsystemjumps");
            while(rs.next())
            {
                SolarSystem from=(SolarSystem)systems.get(Integer.valueOf(rs.getInt(1)));
                SolarSystem to=(SolarSystem)systems.get(Integer.valueOf(rs.getInt(2)));
                jumpsV.addElement(new Jump(from, to));
                if(from!=null && to!=null)
                {
                    Vector v = (Vector) jumpsTable.get(from);
                    if (v == null)
                    {
                        v = new Vector();
                        jumpsTable.put(from, v);
                    }
                    v.addElement(to);
                    v=(Vector) jumpsTable.get(to);
                    if(v==null){
                        v=new Vector();
                        jumpsTable.put(to, v);
                    }
                    v.addElement(from);
                }
            }
            rs.close();
            st.close();

            ///*
            //alliance influence
            Vector<SolarSystem> sovOrig=new Vector<SolarSystem>();
            for(int i=0;i<systemsSov.size();i++){
                sovOrig.addElement(systemsSov.elementAt(i));
            }
            for(int i=0;i<sovOrig.size();i++){
                SolarSystem ss=(SolarSystem)sovOrig.elementAt(i);
                double influence=10.0;
                int level=2;
                //Section for influence multipliers for various bonus factors
                if(ss.stantion){
                    influence*=6;
                    level=1;
                }
                if(ss.sovLevel==2){
                	influence*=1.2;
                }
                if(ss.sovLevel==3){
                	influence*=1.5;
                }
                if(ss.sovLevel==4){
                	influence*=2.0;
                }
                if(ss.conSov==ss.sovereignty.id)
                {
                	influence*=1.5;
                }
                Vector<SolarSystem> v=new Vector<SolarSystem>();
                addInfluence(ss, influence, ss.sovereignty, level, v);
            }
            //NPC
            Hashtable<Integer, Alliance> npc=new Hashtable<Integer, Alliance>();
            Color npcColor=new Color(0,0,0,0);
            st=db.createStatement();
            rs=st.executeQuery("SELECT id, name, systemID, influence FROM npcalliances");
            while(rs.next()){
                int id=rs.getInt(1);
                Alliance al=(Alliance)npc.get(Integer.valueOf(id));
                if(al==null){
                    al=new Alliance();
                    al.id = id;
                    al.name = rs.getString(2);
                    al.nameColor=al.color=npcColor;
                    al.npc=true;
                    npc.put(Integer.valueOf(id), al);
                }
                SolarSystem ss=(SolarSystem)systems.get(Integer.valueOf(rs.getInt(3)));
                if(ss==null)continue;
                double influence=rs.getDouble(4);
                addInfluence(ss, influence, al, 4, new Vector<SolarSystem>());
            }
            jumpsTable=null;
            //drawLegend(false);
            new Thread(this).start();
            //*/
            setSize(1280, 1024);
            setVisible(true);
        }catch(Exception ex){
            ex.printStackTrace();
        }

    }
    public void addInfluence(SolarSystem ss, double value, Alliance al, int level, Vector<SolarSystem> set)
    {
        ss.addInfluence(al, value);
        if(!systemsSov.contains(ss))systemsSov.addElement(ss);
        if(level>=4)return;
        //spread over the jump gates
        Vector arr=(Vector) jumpsTable.get(ss);
        if(arr==null)return;
        for(int i=0;i<arr.size();i++){
            SolarSystem s=(SolarSystem)arr.elementAt(i);
            if(set.contains(s))continue;
            set.addElement(s);
            addInfluence(s, value*0.3, al, level+1, set);
        }
    }

    public void paint(Graphics g) {
        if(g==null)return;
        g.drawImage(img, 5, 20, sWidth/2, 1000, null);
        //g.drawImage(img, 50, 55, null);
        super.paint(g);
    }

    private Color nextColor(){
        int max=0;
        int cr=0,cg=0,cb=0;
        for(int r=0;r<256;r+=4)for(int g=0;g<256;g+=4)for(int b=0;b<256;b+=4){
            if(r+g+b<256 || r+g+b>512)continue;
            int min=1000000000;
            for(int i=0;i<colorTable.size();i++){
                Color c=(Color)colorTable.elementAt(i);
                int dr=r-c.getRed();
                int dg=g-c.getGreen();
                int db=b-c.getBlue();
                int dif=dr*dr+dg*dg+db*db;
                if(min>dif)min=dif;
            }
            if(max<min){
                max=min;
                cr=r;
                cg=g;
                cb=b;
            }
        }
        return new Color(cr, cg, cb, 0x90);
    }


    private class rowState
    {
        public int y, qMax, running;
        public Alliance[]prevRow=new Alliance[0];
        public double[]prevInf=new double[0];
        public boolean[]curBorder=new boolean[0];
        public Alliance[]curRow=new Alliance[0];
    }

    public class calc_row extends Thread
    {
        Hashtable<Alliance, Double> totalInf=new Hashtable<Alliance, Double>();
        final double insensitivity=500;
        final double validInf=0.023;
        public int x_from, x_to, y, qMax, workerID, lasty;
        private double[]prevInf=new double[0];
        private boolean[]curBorder=new boolean[0];
        private Alliance[]curRow=new Alliance[0];      
        private Alliance[]prevRow=null;

        public calc_row (int x_from, int x_to)
        {
            this.x_from = x_from;
            this.x_to = x_to;
        }

        @Override public void run()
        {
            qMax = rowS.qMax;
            curRow = new Alliance[qMax];
            int dx, dy, len2;
            SolarSystem ss;
            Vector<SolarSystem> sSov = new Vector<SolarSystem>();
            int sSize = systemsSov.size();
            Color c;
            for (int i = 0; i < sSize; i++)
            {
                sSov.addElement(systemsSov.elementAt(i));
            }
            //System.out.println(sSize + " systems with sovdata");

            lasty = -1;
            while (true)
            {
                try{ y = intStack.read(workerID); } catch(Exception ex) {}
                // NEVER do the same work twice
                 while (lasty == y)
                 {
                    try{ Thread.sleep(10); y = intStack.read(workerID); } catch(Exception ex) {}
                 }
                //System.out.println(workerID + " got " + y);
                synchronized(rowS)
                {
                    prevRow = rowS.prevRow;
                    prevInf = rowS.prevInf;
                    curBorder = rowS.curBorder;
                }

                for (int x = x_from; x < x_to; x += quarternizer)
                {
                    totalInf.clear();
                    for (int i = 0; i < sSize; i++)
                    {
                        ss = sSov.elementAt(i);
                        dx = x-ss.x;
                        
                        if (dx > 400 || dx < -400) 
                            continue;
                        dy = y-ss.y;
                        
                        if (dy > 400 || dy < -400) 
                            continue;
                        len2 = dx*dx + dy*dy;
                        
                        if (len2>160000)
                            continue;

                        Enumeration en=ss.influence.keys();
                        while (en.hasMoreElements())
                        {
                            Alliance al=(Alliance)en.nextElement();
                            Double d=(Double)totalInf.get(al);
                            double add=((Double)ss.influence.get(al)).doubleValue()/(insensitivity+len2);
                            if (d == null)
                            {
                                totalInf.put(al, Double.valueOf(add));
                            }
                            else
                            {
                                totalInf.put(al, Double.valueOf(d.doubleValue()+add));
                            }
                        }
                    }

                    double max=0.0;
                    Alliance best = null;
                    Enumeration en=totalInf.keys();
                    while (en.hasMoreElements())
                    {
                        Alliance al=(Alliance)en.nextElement();
                        Double d=(Double)totalInf.get(al);
                        if (d.doubleValue()>max)
                        {
                            max=d.doubleValue();
                            best=al;
                        }
                    }

                    int q = x / quarternizer;
                    if (max < validInf) best=null;
                    	
					if (best != null)
					{
						if (best.color == null)
						{
							best.setColor(nextColor());
							saveColor(best);
						}
						best.used = true;
						best.x += x;
						best.y += y;
						best.count++;
						if( (x % sampleRate == 0) && (y % sampleRate) == 0 )
						{
							if (best.npc)
								sovMap[x / sampleRate][y / sampleRate] = null;
							else
							{
								sovMap[x / sampleRate][y / sampleRate] = best;
								//System.out.println("Saved something, I think");
							}
							
						}		
					}
					else if ( (x % sampleRate == 0) && (y % sampleRate) == 0 )
						sovMap[x / sampleRate][y / sampleRate] = null;
					else
						;

                    curRow[q] = best;

                    //draw the prew row point
                    if (y > 0)
                    {
                        Alliance prew = prevRow[q];

                        if (prew != null && !prew.npc)
                        {
                            boolean border = q==0||q==qMax-1||prew!=best||curBorder[q]||prew!=prevRow[q-1]||prew!=prevRow[q+1];

                            int alpha=Math.min(190, (int)(Math.log(Math.log(prevInf[q]+1.0)+1.0)*700));
                            if(!border)
                            {
                                c = new Color(prew.color.getRed(), prew.color.getGreen(), prew.color.getBlue(), alpha);
                            }
                            else
                            {
                                c = new Color(prew.color.getRed(), prew.color.getGreen(), prew.color.getBlue(), Math.max(0x48, alpha));//Changed
                            }
                            synchronized(g)
                            {
                                g.setColor(c);
                                g.fillRect(x, y - quarternizer, quarternizer, quarternizer);
                            }
                        }
                    }
                    prevInf[q] = max;
                    curBorder[q] = (y==0||prevRow[q]!=best);
                }
                //System.out.println(workerID + " pri " + y);
                synchronized(rowS)
                {
                    System.arraycopy(curBorder, x_from, rowS.curBorder, x_from, x_to-x_from);
                    System.arraycopy(prevInf, x_from, rowS.prevInf, x_from, x_to-x_from);
                    System.arraycopy(curRow, x_from, rowS.prevRow, x_from, x_to-x_from);
                }
                try{ intStack.pop(workerID); } catch(Exception ex) {}
                lasty = y;
            }
        }
        // thread end*/
    }

    public class BlockingIntStack {
     private final int[] array;
     private volatile int cnt = 0;
     public BlockingIntStack (int sz)
     { 
         array = new int[sz]; 
      }
     
      synchronized public void pushTimes (int elm, int times) throws InterruptedException {
       while (cnt+times > array.length)
         wait();
       for (int i = 0; i < times; i += 1)
           array[cnt++] = elm;
       notifyAll();
     }
     
     synchronized public int read (int workerID) throws InterruptedException {
       while (cnt == 0)
         wait();
       return(array[cnt-1]);
     } 
     
     synchronized public int pop (int workerID) throws InterruptedException {
       while (cnt == 0)
         wait();
       if (cnt == 1)
         notifyAll();
       return(array[--cnt]);
     } 
    } 
    
    public void run()
    {
        if (1==1)
        {
            final int qMax = sWidth / quarternizer;            

            rowS.qMax = qMax;
            rowS.prevRow = new Alliance[qMax];
            rowS.prevInf = new double[qMax+1];
            rowS.curBorder = new boolean[qMax+1];
            rowS.curRow = new Alliance[qMax];

            calc_row[]rowWorker = new calc_row[sThreads];
            for (int i = 0; i < sThreads; i += 1)
            {
                  rowWorker[i] = new calc_row(sWidth/sThreads*i, sWidth/sThreads*(i+1));
                  rowWorker[i].start();
                  rowWorker[i].workerID = i;
            }

            for (int y = 0; y < sHeight; y += quarternizer)
            {
                paint(getGraphics());
                
                try {intStack.pushTimes(y, sThreads);} catch(Exception ex) {};
            }
            paint(getGraphics());

            for (int x=0; x<sWidth; x+= quarternizer)
            {
                Alliance prew=rowS.prevRow[x/quarternizer];
                if (prew != null)
                {
                    int alpha = Math.min(190, (int)(Math.log(Math.log(rowS.prevInf[x/quarternizer]+1.0)+1.0)*700));
                    g.setColor(new Color(prew.color.getRed(),prew.color.getGreen(),prew.color.getBlue(),alpha));
                    g.fillRect(x, sHeight - quarternizer, quarternizer, quarternizer);
                }
            }
        }
        
        drawLegend(true);
        try{
            javax.imageio.ImageIO.write(img, "png", new java.io.File("influence.png"));
            System.out.println(new Time(System.currentTimeMillis() - time).toString());
            System.out.println("Image saved");
            System.exit(0);
        }catch(Exception ex){
            ex.printStackTrace();
        }
        repaint();
    }
    private void saveColor(Alliance al){
        String r=Integer.toHexString(al.color.getRed());
        while(r.length()<2)r="0"+r;
        String g=Integer.toHexString(al.color.getGreen());
        while(g.length()<2)g="0"+g;
        String b=Integer.toHexString(al.color.getBlue());
        while(b.length()<2)b="0"+b;
        try{
            Statement st=db.createStatement();
            st.executeUpdate("UPDATE evealliances SET color='"+r+g+b+"' WHERE id="+al.id);
            st.close();
        }catch(Exception ex){
            ex.printStackTrace();
        }
    }
    private void drawSovChangeCircles()
    {
    	Statement st;
        Color old = g.getColor();
        g.setColor(new Color(0xff,0xff,0xff, 0xbf));
    	try {
			st = db.createStatement();
			ResultSet rs=st.executeQuery("SELECT r.x, r.z from sovchangelog s LEFT JOIN mapsolarsystems r ON s.systemID=r.solarsystemID;");
			while(rs.next()){
            SolarSystem ss=new SolarSystem();
            ss.x=(int)(rs.getDouble(1)/scale)+sWidth/2+sX;
            ss.y=(int)(rs.getDouble(2)/scale)+sHeight/2+sY;
            g.drawOval(ss.x-4, ss.y-4, 8, 8);
        }
        rs.close();
        st.close();
        g.setColor(old);
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }
    
	/*Draekas Addition.  This method uses the sovMap 2D array which was filled during Run() with influence information
	and builds a vector of alliance names and coordinates.*/
	private  int eatAlliance(TitleVectorItem titleItem, int xCoordinate, int yCoordinate)
	{
		
		int sumX, sumY, counter = 0;
		//System.out.println(callCount++);
		if(sovMap[xCoordinate][yCoordinate] == null)
			return 0;
		else if(!sovMap[xCoordinate][yCoordinate].name.equals(titleItem.allianceName))
		{
			//System.out.println("different alliance seen");
			return 0;
		}
		else
		{
			//System.out.println("in eatAlliance positive " + sovMap[xCoordinate][yCoordinate].name + " = " + titleItem.allianceName );
			titleItem.xCoord += xCoordinate * sampleRate;// * sampleRateModifier;
			titleItem.yCoord += yCoordinate * sampleRate;// * sampleRateModifier;
			//titleItem.pixelCount++;		
			counter++;
			sovMap[xCoordinate][yCoordinate] = null;
			for(int y = -1; y <= 1; y += 1)
			{
				for(int x = -1; x <= 1; x += 1)
				{
					sumX = x + xCoordinate;
					sumY = y + yCoordinate;

					if(x == 0 && y == 0)
						;
					else if(sumX < 0 || sumX >= sWidth/sampleRate)
						;
					else if(sumY < 0 || sumY >= sHeight/sampleRate)
						;
					else
					{							
						//System.out.println("calling more eat alliances and stuff");						
						counter += eatAlliance(titleItem, sumX, sumY);						
					}
				}
			}
			//System.out.println("returning counter = " + counter);
			return counter;
		}
	
	}

	/*Draekas Addition.  Draws the Alliance titles on the map
	*/
	/*Draekas Addition.  Draws the Alliance titles on the map
	*/
	private void drawAllianceTitles()
	{
		TitleVectorItem temp;
		Font aliFont;
		int ax, ay;
		Rectangle2D r;
		Font defaultFont=g.getFont();

		while(!titleList.isEmpty())
		{
			temp = (TitleVectorItem) titleList.remove(0);
			
		
			ax = ( temp.xCoord / temp.pixelCount + sampleRate/2);
			ay = ( temp.yCoord / temp.pixelCount + sampleRate/2);
			//This next line controls the alliance name text size, if names are the wrong size, mess with this
			aliFont=new Font("Verdana", Font.BOLD, (int)(Math.sqrt(temp.pixelCount)* quarternizer /3)+8);
			//System.out.println("Writing at "+ ax + ":" + ay + "With font size" + aliFont.getSize());

			//System.out.println(temp.allianceName + ": " + temp.pixelCount + " - " + aliFont.getSize());
			g.setFont(aliFont);
			r=aliFont.getStringBounds(temp.allianceName, g.getFontRenderContext());
			g.setColor(Color.BLACK);
			g.drawString(temp.allianceName, ax-(int)r.getWidth()/2-1, ay+10);
			g.drawString(temp.allianceName, ax-(int)r.getWidth()/2+1, ay+10);
			g.drawString(temp.allianceName, ax-(int)r.getWidth()/2, ay+10-1);
			g.drawString(temp.allianceName, ax-(int)r.getWidth()/2, ay+10+1);
			g.setColor(temp.nameColor);
			g.drawString(temp.allianceName, ax-(int)r.getWidth()/2, ay+10);
			g.setFont(defaultFont);

		}
	}
	
	
	/*Draekas Addition.  Fills the Alliance title vector needed for drawAllianceTitles.
	*/
	private  void fillAllianceTitleVector()
	{
		TitleVectorItem temp;

		for(int y = 0; y < sHeight/sampleRate; y++)
		{
			for(int x = 0; x < sWidth/sampleRate; x++)
			{
				if (sovMap[x][y] != null)
				{
					temp = new TitleVectorItem();
					
					//System.out.println(sovMap[x][y].name);
					temp.allianceName = sovMap[x][y].name;
					temp.nameColor = sovMap[x][y].nameColor;
					temp.pixelCount = eatAlliance(temp, x, y);
					titleList.add(temp);
					//System.out.println("new vector at (" + x + " , " + y + ")" + " and count=" + temp.pixelCount);
					temp = null;
				}
			}
		}

	}

    
    private void drawLegend(boolean checkUsed){
        //draw jumps
        for(int i=0;i<jumpsV.size();i++)((Jump)jumpsV.elementAt(i)).draw();
        //draw systems
        Enumeration en=systems.elements();
        while(en.hasMoreElements())((SolarSystem)en.nextElement()).draw();
        paint(getGraphics());
        drawSovChangeCircles();
        try{
        	Statement st=db.createStatement();
        	ResultSet rs;
            
        	//region names
            g.setColor(new Color(0xff,0xff,0xff, 0xB0));
            Font font=g.getFont();
            st=db.createStatement();
            rs=st.executeQuery("SELECT r.x, r.z, r.regionName FROM mapsolarsystems s LEFT JOIN mapregions r ON r.regionID=s.regionID /*WHERE s.allianceID<>0*/ GROUP BY r.regionID");
            while(rs.next()){
                int x=(int)(rs.getDouble(1)/scale)+sWidth/2+sX;
                int y=(int)(rs.getDouble(2)/scale)+sHeight/2+sY;
                String s=rs.getString(3);
                Rectangle2D r=font.getStringBounds(s, g.getFontRenderContext());
                g.drawString(s, x-(int)r.getWidth()/2, y+10);
            }
            
            st=db.createStatement();
        	st.executeUpdate("UPDATE evealliances SET onMap=0");
            st.close();
            //legend
            //int y=17;
            //int x=sWidth-210;
            g.setColor(Color.GREEN);
            //g.drawString("LEGEND", x+17, y-5);
            g.setColor(Color.DARK_GRAY);
            //Set the font bold for a moment for the map title
            g.setFont(new Font(g.getFont().getFontName(),Font.BOLD,g.getFont().getSize()));
            g.drawString("EVE Null-Sec Player Influence Map", 4, 17);
            //Now set the font back to normal, although I have no idea where the initial settings are coming from
            g.setFont(font);
            g.drawString("Generated by Verite Rendition", 4, 35);
            //Out of respect for the author, we leave Paladin's name on the map
            g.drawString("Created by Paladin Vent", 4, 53);
            g.drawString("Using sovereignty data on "+soveregntyDate, 4, 71);
            //g.drawString("Station data provided by EVE Strategic Maps", 0, 71);
            Font defaultFont=g.getFont();

			//Draekas Addition***********************************************
			fillAllianceTitleVector();
			drawAllianceTitles();
			
            Font small=new Font("MS Sans Serif",0,9);
            g.setFont(small);
            g.setColor(Color.WHITE);
            int y=106;
            g.drawString("Sov. Lost",35,y);
            g.drawString("Sov. Gain",135,y);
            g.drawString("System",208,y);
            g.drawString("Region",250,y);
            y+=15;
            st=db.createStatement();
            Shape defClip=g.getClip();
            String sql=
                    "SELECT l.fromAllianceID, l.toAllianceID, s.solarSystemName, r.regionName, s.stantion " +
                    "FROM sovchangelog l " +
                    //"LEFT JOIN evealliances a1 ON l.fromAllianceID=a1.id " +
                    //"LEFT JOIN evealliances a2 ON l.toAllianceID=a2.id " +
                    "LEFT JOIN mapsolarsystems s ON s.solarSystemID=l.systemID " +
                    "LEFT JOIN mapregions r ON s.regionID=r.regionID " +
                    //"WHERE l.date='"+soveregntyDate+"' " +
                    //"WHERE l.date='2007-03-06' " +
                    "ORDER BY r.z, r.x";
            System.out.println(sql);
            rs=st.executeQuery(sql);
            Color cStantion=new Color(0, 0, 0x40);
            while(rs.next()){
                //String fromS=rs.getString(1), toS=rs.getString(2);
                String system=rs.getString(3), region=rs.getString(4);
                if(rs.getInt(5)==1){
                    g.setClip(5,y-50,298,100);
                    g.setColor(cStantion);
                    g.fillRect(5, y-10, 290, 13);
                }
                //if(stantion)system="OP:"+system;
                Alliance from=(Alliance)alliances.get(Integer.valueOf(rs.getInt(1)));
                if(from!=null){
                    g.setClip(5,y-50,98,100);
                    g.setColor(from.nameColor);
                    g.drawString(from.name, 5, y);
                }
                Alliance to=(Alliance)alliances.get(Integer.valueOf(rs.getInt(2)));
                if(to!=null){
                    g.setClip(105,y-50,98,100);
                    g.setColor(to.nameColor);
                    g.drawString(to.name, 105, y);
                }
                g.setColor(starColor);
                g.setClip(205,y-50,40,100);
                g.drawString(system, 205, y);
                g.setColor(Color.WHITE);
                g.setClip(246,y-50,50,100);
                g.drawString(region, 246, y);
                y+=15;
            }
            g.setClip(defClip);
            g.setFont(defaultFont);
            g.setColor(Color.DARK_GRAY);
            int y2=110;
            g.drawLine(5,y2,295,y2);
            g.drawLine(104,y2-15,104,y-10);
            g.drawLine(204,y2-15,204,y-10);
            g.drawLine(245,y2-15,245,y-10);
            rs.close();
            st.close();
 
        }catch(Exception ex){
            ex.printStackTrace();
        }
    }
    private static final Color starColor=new Color(0xB0, 0xB0, 0xFF);

    private class SolarSystem{
        int x, y;
        int id, constelationID, RegionID, sovLevel, conSov;
        Alliance sovereignty;
        Hashtable<Alliance, Double> influence=new Hashtable<Alliance, Double>();
        boolean stantion;
        void addInfluence(Alliance al, double value){
            Double inf=(Double)influence.get(al);
            if(inf==null){
                influence.put(al, Double.valueOf(value));
            }else{
                influence.put(al, Double.valueOf(value+inf.doubleValue()));
            }
        }
        void draw(){
            if(sovereignty!=null && sovereignty.color==null){
                sovereignty.setColor(nextColor());
                saveColor(sovereignty);
            }
            g.setColor(sovereignty!=null?sovereignty.starColor:starColor);
            if(stantion){
                g.fillOval(x-1, y-1, 2, 2);
                g.drawRect(x-2,y-2, 4, 4);
            }else if (sovereignty!=null){
                g.fillOval(x-2, y-2, 4, 4);
            }else{
                g.fillOval(x-1, y-1, 2, 2);
            }
        }
    }
    private class Alliance{
        int id;
        String name;
        Color color=null, nameColor=null, starColor=null;
        boolean used=false, npc=false;
        long x=0, y=0;
        long count=0;
        void setColor(Color c){
            color=new Color(c.getRGB());
            float[] hsb=Color.RGBtoHSB(color.getRed(), color.getGreen(), color.getBlue(), null);
            hsb[2]=1;
            nameColor=new Color(Color.HSBtoRGB(hsb[0], hsb[1], hsb[2]));
            hsb[1]=Math.max(0.2f, hsb[1]*.7f);
            starColor=new Color(Color.HSBtoRGB(hsb[0], hsb[1], hsb[2]));
            colorTable.addElement(color);
        }
    }
    /*private class MapPoint{
        Alliance claim;
        double inluence;
    }*/
    private class Jump{
        SolarSystem from, to;
        private final Color sJump=new Color(0, 0, 0xFF, 0x30);
        private final Color cJump=new Color(0xFF, 0, 0, 0x30);
        private final Color rJump=new Color(0xFF, 0, 0xFF, 0x30);
        Jump(SolarSystem from, SolarSystem to){
            this.from=from;
            this.to=to;
        }
        void draw(){
            if(from.constelationID==to.constelationID){
                g.setColor(sJump);
            }else if(from.RegionID==to.RegionID){
                g.setColor(cJump);
            }else{
                g.setColor(rJump);
            }
            g.drawLine(from.x, from.y, to.x, to.y);
        }
    }
	/*Draekas Addition.  This class is used for the alliance name
	vector.
	*/
	private class TitleVectorItem
	{
		int xCoord, yCoord;
		int pixelCount;
		String allianceName;
		Color nameColor;
	};
}
